require 'rexml/document'

module GenomeLib
  class VirtGuest
    attr_reader :name, :uuid, :hostname, :ipaddress, :appliance, :status, :arch, :memsize

    def initialize(name, uuid, hostname, ipaddress, appliance, status, arch, memsize)
      @name = name
      @uuid = uuid
      @hostname = hostname
      @ipaddress = ipaddress
      @appliance = appliance
      @status = status
      @arch = arch
      @memsize = memsize
    end

    def running?
      @status == "on"
    end
  end

  class Nodeinfo
    attr_reader :cpu_model, :memsize

    def initialize(cpu_model, memsize)
      @cpu_model = cpu_model
      @memsize = memsize.to_i
    end
  end

  class Virsh
    include REXML

    VIRSH = "virsh"

    def list(options={})
      defaults = {:all => false}
      opts = defaults.merge(options)
      
      args = []
      args << "--all" if opts[:all] 
      parse_list(run_command('list', args))
    end

    def nodeinfo
      out = run_command('nodeinfo', [])
      Nodeinfo.new(parse_cpu_model(out), parse_mem_size(out))
    end

    def start(uuid)
      name = get_name(uuid)
      args = []
      args << name
      run_command('start', args)
      return $?.exitstatus
    end

    def shutdown(uuid)
      name = get_name(uuid)
      args = []
      args << name
      run_command('shutdown', args)
      return $?.exitstatus
    end

    def guest_mem(dom_name)
      guest_xml(dom_name).root.elements["/domain/memory"].text.to_i rescue 0
    end

    def autostart(opts={})
      name = opts[:domname].nil? ? get_name(opts[:uuid]) : opts[:domname]

      # popen3 doesn't capture the exist status because of the way it spawns
      # the child process
      IO.popen("virsh autostart #{name} 2>&1") do |io|
        yield(io) if block_given?
      end

      if $?.exitstatus != 0
        # We're probably running Xen.  
        if File.exist?("/etc/xen")
          FileUtils.rm_rf("/etc/xen/auto/#{name}") # in case this machine already existed
          FileUtils.ln_s("/etc/xen/#{name}", "/etc/xen/auto/#{name}")
        end
      end
    end

    def autostart_disable(opts={})
      name = opts[:domname].nil? ? get_name(opts[:uuid]) : opts[:domname]

      # popen3 doesn't capture the exist status because of the way it spawns
      # the child process
      IO.popen("virsh autostart --disable #{name} 2>&1") do |io|
        yield(io) if block_given?
      end

      if $?.exitstatus != 0
        # We're probably running Xen.  
        if File.exist?("/etc/xen")
          FileUtils.rm_rf("/etc/xen/auto/#{name}") # in case this machine already existed
        end
      end
    end

    private 

    def guest_xml(dom_name)
      Document.new(run_command_raw("dumpxml", [dom_name]))
    end

    def run_command_raw(cmd, *args)
      `#{VIRSH} #{cmd} #{args.join(" ")}`
    end

    def run_command(cmd, *args)
      `#{VIRSH} #{cmd} #{args.join(" ")}`.split("\n")
    end

    def parse_list(output)
      # Chop off the first two lines
      guests = output[2..-1].map do |line|
        line =~ /\s*(?:\d+|[-])\s([-0-9A-Za-z_\.]+)\s+(.*)$/
        name = $1
        next if name == "Domain-0" # This causes 'nil' to be returned

        # TODO: It would be better to use a boolean here
        status = case $2 
          when /running|blocked/ then "on" 
          when /no state/ then "unknown"
          else "off"
        end

        VirtGuest.new(name, get_uuid(name), nil, nil, nil, status, nil, guest_mem(name))
      end.reject {|g| g.nil?}

      return guests
    end

    def parse_cpu_model(output)
      parse_info(output, "CPU model", /\w+/)
    end

    def parse_mem_size(output)
      parse_info(output, "Memory size", /\d+/)
    end

    def parse_info(output, key, regex)
      output.find {|l| l =~ /^#{key}:\s+(#{regex}).*$/}
      return $1
    end
   
    def get_uuid(name)
      `virsh domuuid #{name}`.strip
    end

    # Get the name of a domain based
    # on the dom id or the dom uuid
    # See "virsh help domname"
    def get_name(id)
      `virsh domname #{id}`.strip
    end
  end
end
