# Copyright (C) 2008 Red Hat, Inc

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# a long with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# These are required by the Rakefile.  The tests should probably run in a separate process.
$:.unshift(File.dirname(__FILE__) + "/../lib")
require 'genome-lib'
require 'tempfile'
require 'rubygems'
require 'test/unit'
require 'mocha'
require 'active_record'

include GenomeLib 


module DataForTest

  def cobbler_profiles
    [
      {"name" => "Profile_1__Appliance", "virt_file_size" => "", "virt_ram" => ""},
      {"name" => "Profile_2__Guest", "virt_file_size" => "20", "virt_ram" => "256"},
      {"name" => "Profile_3__Guest", "virt_file_size" => "30", "virt_ram" => "2048"}
    ]
  end
end

class TestClassAccessorsWithDefaults < Test::Unit::TestCase
  def setup
    @foo_klass = Class.new do
      include ClassAccessorsWithDefaults
      cattr_accessor :bar, "buz"
    end

    @f = @foo_klass.new
  end

  def test_class_accessor_become_instance_readers
    assert_equal("buz", @foo_klass.bar)
    assert_equal("buz", @f.bar)
  end

  def test_exception_thrown_for_bad_cattr
    assert_raise(NoMethodError) { @f.opps }
  end
end

class TestGenomeConfig < Test::Unit::TestCase
  include DataForTest
  def setup
    @conf_file_text = <<-TEXT
---
appliance: foo.bar.example.com
machine_type: buz
this isn't a good idea: boo
    TEXT
    
    @conf_file = Tempfile.new("genome-config")
    @conf_file.puts(@conf_file_text)
    @conf_file.flush
  end

  def test_conf_file_attr
    config = GenomeConfig.new(@conf_file.path)
    assert_equal(@conf_file.path, config.conf_file)
  end

  def test_read_values_with_string_and_symbol
    config = GenomeConfig.new(@conf_file.path)
    assert_equal("foo.bar.example.com", config["appliance"])
    assert_equal("foo.bar.example.com", config[:appliance])

    assert_equal("boo", config["this isn't a good idea"])
  end

  def test_read_non_existent
    config = GenomeConfig.new(@conf_file.path)
    assert_equal(nil, config["this_doesnt_exist"])
  end

  def test_default_values
    config = GenomeConfig.new(@conf_file.path)
    assert_equal(":memory:", config[:cloudhost_db])
  end

  def test_empty_config_if_file_not_found
    assert_equal(nil, GenomeConfig.new("/foo/bar")[:should_be_nil])
  end

  def test_invalid_yaml_raises_error
    File.expects(:exist?).returns(true)
    File.expects(:read).returns("\t\t")
    assert_raise RuntimeError do
      GenomeConfig.new(@conf_file.path)
    end
  end

  def test_default_config_file_location
    File.expects(:exist?).returns(true)
    File.expects(:read).returns(@conf_file_text)
    assert_nothing_raised {GenomeConfig.new}
  end

  def test_global_accessor
    assert_not_nil(Genome)
    assert_kind_of(GenomeConfig, Genome)
    assert_equal(nil, Genome[:should_be_nil])
  end
end

class TestClassEvictor < Test::Unit::TestCase
  NOW = Time.now

  class TestRecord < ActiveRecord::Base; end
  class TestSchema < ActiveRecord::Migration
    def self.up
      create_table :test_records do |t|
        t.column :id,           :integer,  :null => false
        t.column :last_updated, :datetime, :null => false
      end
 
      10.times do |i|
        TestRecord.create :last_updated => NOW - (i * 60)
      end
    end

    def self.down
      drop_table :test_records
    end
  end

  def setup
    ActiveRecord::Base.establish_connection(
      :adapter => "sqlite3",
      :database  => ":memory:"
    )

    TestSchema.migrate(:up)
  end

  def teardown
    TestSchema.migrate(:down)
    ActiveRecord::Base.remove_connection
  end

  def test_eviction
    Time.stubs(:now).returns(NOW) # Just to avoid any possible timing problems
    assert_equal(10, TestRecord.find(:all).size)
    Evictor.new(TestRecord, 120).run
    assert_equal(3, TestRecord.find(:all).size)
  end

  def test_optional_logger
    Time.stubs(:now).returns(NOW) # Just to avoid any possible timing problems
    assert_equal(10, TestRecord.find(:all).size)
    e = Evictor.new(TestRecord, 120)
    e.expects(:log).times(7)
    e.run 
  end
end

class TestGenomeAppliance < Test::Unit::TestCase
  include DataForTest
  def setup
    @cobbler_profiles = [
      {"name" => "Profile_1__Appliance", "virt_file_size" => "", "virt_ram" => ""},
      {"name" => "Profile_2__Guest", "virt_file_size" => "20", "virt_ram" => "256"},
      {"name" => "Profile_3__Guest", "virt_file_size" => "30", "virt_ram" => "2048"}
    ]
    @guest_profiles = @cobbler_profiles.delete_if{|p|p["name"].include?"__Appliance"} 
    @genome_fqdn = "server.domain.com"
  end

  def test_guest_appliances
    genome = Appliances::Genome.new(@genome_fqdn)
    genome.expects(:cobbler_profiles).returns(@cobbler_profiles)
    assert_equal(genome.guest_profiles, @guest_profiles)
  end
end
